//
// errors.h
//
// defines constants for error-handling and exceptions
//
#ifndef ERRORS_H
#define ERRORS_H

#include <string>
#include "lang.h"

enum { 
	E_USER = -2, 		// User-specified error text
	E_UNKNOWN = -1, 	// Unknown error occured
	E_SUCCESS = 0, 		// No error
	E_NOT_IMPL,             // Requested feature not implemented
	//-- tree errors
	E_DIFF_BOOL,		// Attempt to diff boolean
	E_DIFF_IFTHENELSE,	// Attempt to diff if-then-else
	E_DIFF_POWNOTINDEP,  // power factor was not independent of diff-var
	E_INVALID_SUBTYPE,	// Invalid subtype in expression
	E_DIV_BY_ZERO,		// Division by zero detected!
	E_IFCOND_NOT_BOOL,	// If-condition is not of type bool!
	E_EVAL_NOTCONST,	// Evaluation done on metaexpr
	E_EVAL_NORULE,		// No rule available to calculate
	E_EVAL_BOOL,		// Cannot evaluate boolean expression
	E_EVAL_REAL,		// Cannot evaluate real expression
	E_EVAL_NEGLN,     // attempt to calculate negative value for LN
	E_EVAL_POWNEGBASE, // attempt to raise negative value to real power.
	//-- scanner errors
	E_SC_INVCHAR,           //
	E_SC_INVNOT,   			//
	E_SC_INVDEF,		    //':' blev ikke efterfulgt af et '='
	E_SC_DECPOINT,          //
	//-- parser errors
	E_PS_MISSINGTOK,        //
	E_PS_SYNTAX,            //
	E_PS_REDEF,             // Ident redefined
	E_PS_UNDEFID,           // Ident undefined, type is unknown at parse-time
	E_PS_REDEF_CONST,       // This error uccours if a const is redefined as a meta-expression
	E_PS_MISSINGEXP,			// Missing argument for unary function
	E_PS_UNBOUND,
	E_PS_ENDOFINPUT,

};

class Exception {
	public:
	
		Exception( int errorcode = -1, string msg = "" ): ecode(errorcode), message(msg) {};
		~Exception() {};
	
		string getMsg() {
		        string s = "";

			switch (ecode) {
			case E_USER: return message ; 
			case E_SUCCESS: s = lang_get(1); break;			
			case E_UNKNOWN: s = lang_get(2); break;
			case E_NOT_IMPL: s = lang_get(3); break;
			//-- tree errors
			case E_DIFF_BOOL: s = lang_get(4); break;
			case E_DIFF_IFTHENELSE: s = lang_get(5); break;
			case E_DIFF_POWNOTINDEP: s = lang_get(EX_DIFF_POWNOTINDEP); break;
			case E_INVALID_SUBTYPE: s = lang_get(6); break;
			case E_DIV_BY_ZERO: s = lang_get(7); break;
			case E_IFCOND_NOT_BOOL: s = lang_get(8); break;
			case E_EVAL_NOTCONST: s = lang_get(9); break;
			case E_EVAL_BOOL: s = lang_get(10); break;
			case E_EVAL_REAL: s = lang_get(11); break;
			case E_EVAL_NORULE: s = lang_get(12); break;
			case E_EVAL_NEGLN: s = lang_get(26); break;
			case E_EVAL_POWNEGBASE: s = lang_get(EX_EVAL_POWNEGBASE); break;
			//-- scanner errors
			case E_SC_INVCHAR: s = lang_get(13); break;
			case E_SC_INVNOT: s = lang_get(14); break;
			case E_SC_INVDEF: s = lang_get(15); break;
			case E_SC_DECPOINT: s = lang_get(16); break;
			  //-- parser errors
			case E_PS_MISSINGTOK: s = lang_get(17); break;
			case E_PS_SYNTAX: s = lang_get(18); break;
			case E_PS_UNDEFID: s = lang_get(19); break;
			case E_PS_REDEF: s = lang_get(20); break;
			case E_PS_REDEF_CONST: s = lang_get(21); break;
			case E_PS_UNBOUND: s = lang_get(24); break;
			case E_PS_ENDOFINPUT: s = lang_get(25); break;

			default: s = lang_get(99); break;
			}
			if (message!="")
			  s += "\n\t-> "+message;
			return s+"\a";
		}
		
	protected:
		string message;
		int ecode;
};
	
			

#endif	






